'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

import { mapAsync } from './AsyncUtils';
import warning from './routerWarning';

function getComponentsForRoute(nextState, route, callback) {
  if (route.component || route.components) {
    callback(null, route.component || route.components);
    return;
  }

  var getComponent = route.getComponent || route.getComponents;
  if (getComponent) {
    var _ret = (function () {
      // By assumption, location is a POJSO, and can be spread into the router
      // state. The merging means that we can't use deprecateObjectProperties.
      var location = nextState.location;

      var nextStateWithLocation = _extends({}, nextState, location);

      if (process.env.NODE_ENV !== 'production') {
        if (typeof Proxy === 'function') {
          nextStateWithLocation = new Proxy(nextStateWithLocation, {
            get: function get(target, name) {
              if (Object.prototype.hasOwnProperty.call(location, name)) {
                process.env.NODE_ENV !== 'production' ? warning(false, 'Accessing location properties from the first argument to `getComponent` and `getComponents` is deprecated. That argument is now the router state rather than the location. To access the location, use `state.location`.') : undefined;
              }
              return target[name];
            }
          });
        }
      }

      getComponent(nextStateWithLocation, callback);
      return {
        v: undefined
      };
    })();

    if (typeof _ret === 'object') return _ret.v;
  }

  callback();
}

/**
 * Asynchronously fetches all components needed for the given router
 * state and calls callback(error, components) when finished.
 *
 * Note: This operation may finish synchronously if no routes have an
 * asynchronous getComponents method.
 */
function getComponents(nextState, callback) {
  mapAsync(nextState.routes, function (route, index, callback) {
    getComponentsForRoute(nextState, route, callback);
  }, callback);
}

export default getComponents;